import numpy as np
import os
import torch
from scipy.stats import f
from scipy.sparse import csr_matrix
import scanpy as sc
import scanpy.external as sce
from anndata import AnnData
import cellrank as cr
from cellrank.tl.kernels import VelocityKernel

import torch
import torch.nn as nn

def construct_dag(joint_feature_embeddings,iroot,n_neighbors=15,pseudotime_algo='dpt'):
	
	"""Constructs the adjacency matrix for a DAG.
	Parameters
	----------
	joint_feature_embeddings: 'numpy.ndarray' (default: None)
		Matrix of low dimensional embeddings with rows corresponding
		to observations and columns corresponding to feature embeddings
		for constructing a DAG if a custom DAG is not provided.
	iroot: 'int' (default: None)
		Index of root cell for inferring pseudotime for constructing a DAG 
		if a custom DAG is not provided.
	n_neighbors: 'int' (default: 15)
		Number of nearest neighbors to use in constructing a k-nearest
		neighbor graph for constructing a DAG if a custom DAG is not provided.
	pseudotime_algo: {'dpt','palantir'} 
		Pseudotime algorithm to use for constructing a DAG if a custom DAG 
		is not provided. 'dpt' and 'palantir' perform the diffusion pseudotime
		(Haghverdi et al., 2016) and Palantir (Setty et al., 2019) algorithms, 
		respectively.
	"""

	pseudotime,knn_graph = infer_knngraph_pseudotime(joint_feature_embeddings,iroot,
		n_neighbors=n_neighbors,pseudotime_algo=pseudotime_algo)
	dag_adjacency_matrix = dag_orient_edges(knn_graph,pseudotime)

	return dag_adjacency_matrix

def infer_knngraph_pseudotime(joint_feature_embeddings,iroot,n_neighbors=15,pseudotime_algo='dpt'):

	adata = AnnData(joint_feature_embeddings)
	adata.obsm['X_joint'] = joint_feature_embeddings
	adata.uns['iroot'] = iroot

	if pseudotime_algo == 'dpt':
		sc.pp.neighbors(adata,use_rep='X_joint',n_neighbors=n_neighbors)
		sc.tl.dpt(adata)
		adata.obs['pseudotime'] = adata.obs['dpt_pseudotime'].values
		knn_graph = adata.obsp['distances'].astype(bool).astype(float)
	elif pseudotime_algo == 'palantir':
		sc.pp.neighbors(adata,use_rep='X_joint',n_neighbors=n_neighbors)
		sce.tl.palantir(adata, knn=n_neighbors,use_adjacency_matrix=True,
			distances_key='distances')
		pr_res = sce.tl.palantir_results(adata,
			early_cell=adata.obs.index.values[adata.uns['iroot']],
			ms_data='X_palantir_multiscale')
		adata.obs['pseudotime'] = pr_res.pseudotime
		knn_graph = adata.obsp['distances'].astype(bool).astype(float)

	return adata.obs['pseudotime'].values,knn_graph

def dag_orient_edges(adjacency_matrix,pseudotime):

	A = adjacency_matrix.astype(bool).astype(float)
	D = -1*np.sign(pseudotime[:,None] - pseudotime).T
	D = (D == 1).astype(float)
	D = (A.toarray()*D).astype(bool).astype(float)

	return D

def construct_S(D):
        
    S = D.clone()
    D_sum = D.sum(0)
    D_sum[D_sum == 0] = 1
    
    S = (S/D_sum)
    S = S.T
    
    return S

def normalize_adjacency(D):
        
    S = D.clone()
    D_sum = D.sum(0)
    D_sum[D_sum == 0] = 1
    
    S = (S/D_sum)
    
    return S

def load_multiome_data(data_dir,dataset,sampling=None,preprocess=True):

	if sampling == 'geosketch':
		atac_adata = sc.read(os.path.join(data_dir,
			'{}.atac.sketch.h5ad'.format(dataset)))
		rna_adata = sc.read(os.path.join(data_dir,
			'{}.rna.sketch.h5ad'.format(dataset)))
	elif sampling == 'uniform':
		atac_adata = sc.read(os.path.join(data_dir,
			'{}.atac.uniform.h5ad'.format(dataset)))
		rna_adata = sc.read(os.path.join(data_dir,
			'{}.rna.uniform.h5ad'.format(dataset)))
	else:
		atac_adata = sc.read(os.path.join(data_dir,
			'{}.atac.h5ad'.format(dataset)))
		rna_adata = sc.read(os.path.join(data_dir,
			'{}.rna.h5ad'.format(dataset)))

	if preprocess:

		# scale by maximum 
		# (rna already normalized by library size + log-transformed)
		X_max = rna_adata.X.max(0).toarray().squeeze()
		X_max[X_max == 0] = 1
		rna_adata.X = csr_matrix(rna_adata.X / X_max)

		# atac: normalize library size + log transformation
		sc.pp.normalize_total(atac_adata,target_sum=1e4)
		sc.pp.log1p(atac_adata)

	return rna_adata,atac_adata

def seq2dag(N):
    A = torch.zeros(N, N)
    for i in range(N - 1):
        A[i][i + 1] = 1
    return A

def activation_helper(activation, dim=None):
    if activation == 'sigmoid':
        act = nn.Sigmoid()
    elif activation == 'tanh':
        act = nn.Tanh()
    elif activation == 'relu':
        act = nn.ReLU()
    elif activation == 'leakyrelu':
        act = nn.LeakyReLU()
    elif activation is None:
        def act(x):
            return x
    else:
        raise ValueError('unsupported activation: %s' % activation)
    return act

def calculate_AX(A,X,lag):

    if A == "linear":
        A = seq2dag(X.shape[1])
    S = construct_S(A)

    ax = []
    cur = S
    for _ in range(lag):
        ax.append(torch.matmul(cur, X))
        cur = torch.matmul(S, cur)
        for i in range(len(cur)):
            cur[i][i] = 0

    return torch.stack(ax)


#!/usr/bin/env python

### Authors: Anish Mudide (amudide), Alex Wu (alexw16), Rohit Singh (rs239)
### 2022
### MIT Licence
###


import numpy as np
import torch
import torch.nn as nn
from copy import deepcopy
import time

from .models import *
from .utils import *

def train_model(config, checkpoint_dir = None):

	AX = config["AX"]
	Y = config["Y"]

	method = config["method"]

	trial_no = config["trial"]
	lr = config["lr"]
	lam = config["lam"]
	lam_ridge = config["lam_ridge"]
	penalty = config["penalty"]
	lag = config["lag"]
	hidden = config["hidden"]
	max_iter = config["max_iter"]
	device = config["device"]
	lookback = config["lookback"]
	check_every = config["check_every"]
	verbose = config["verbose"]
	tol = config['tol']
	dynamics = config['dynamics']
		
	gc_dir = config['gc_dir']
	dir_name = config['dir_name']

	np.random.seed(trial_no)
	torch.manual_seed(trial_no)


	file_name = '{}.trial{}.lam{}.ridge{}.h{}.{}.lag{}.{}'.format(method,trial_no,lam,
				lam_ridge,hidden[0],penalty,lag,dynamics)
	gc_path1 = os.path.join(gc_dir,dir_name,file_name + '.pt')
	gc_path2 = os.path.join(gc_dir,dir_name,file_name + '.ignore_lag.pt')

	if not os.path.exists(gc_path1) and not os.path.exists(gc_path2):
		
		num_regs = AX.shape[-1]
		num_targets = Y.shape[1]

		vmlp = VeloramaMLP(num_targets, num_regs,lag=lag, hidden=hidden, device=device, activation='relu')
		vmlp.to(device)

		AX = AX.to(device)
		Y = Y.to(device)

		#if torch.cuda.device_count() > 1:
		#    vmlp = nn.DataParallel(vmlp)
			
		'''Train model with ISTA.'''
		lag = vmlp.lag
		# p = Y.shape[1]
		loss_fn = nn.MSELoss(reduction='none')
		train_loss_list = []

		# For early stopping.
		best_it = None
		best_loss = np.inf
		best_model = None

		# Calculate smooth error.
		loss = loss_fn(vmlp(AX),Y).mean(0).sum()
		# loss = sum([loss_fn(vmlp.networks[i](AX), Y[:,i]) for i in range(Y.shape[1])])

		ridge = torch.sum(torch.stack([ridge_regularize(net, lam_ridge) for net in vmlp.networks]))
		smooth = loss + ridge

		variable_usage_list = []
		loss_list = []

		# For early stopping.
		train_loss_list = []
		best_it = None
		best_loss = np.inf
		best_model = None

		for it in range(max_iter):

			start = time.time()
			# Take gradient step.
			smooth.backward()
			for param in vmlp.parameters():
				param.data = param - lr * param.grad

			# Take prox step.
			if lam > 0:
				for net in vmlp.networks:
					prox_update(net, lam, lr, penalty)

			vmlp.zero_grad()

			# Calculate loss for next iteration.
			loss = loss_fn(vmlp(AX),Y).mean(0).sum()
			# loss = sum([loss_fn(vmlp.networks[i](AX), Y[:,i]) for i in range(Y.shape[1])])
			ridge = torch.sum(torch.stack([ridge_regularize(net, lam_ridge) for net in vmlp.networks]))
			smooth = loss + ridge

			# Check progress.
			if (it + 1) % check_every == 0:

				nonsmooth = torch.sum(torch.stack([regularize(net, lam, penalty) for net in vmlp.networks]))
				mean_loss = (smooth + nonsmooth).detach()/Y.shape[1]

				variable_usage = torch.mean(vmlp.GC(ignore_lag=False).float())
				variable_usage_list.append(variable_usage)
				loss_list.append(mean_loss)

				# Check for early stopping.
				if mean_loss < best_loss:
					best_loss = mean_loss
					best_it = it
					best_model = deepcopy(vmlp)

					if verbose:
						print('Lam={}: Iter {}, {} sec'.format(lam,it+1,np.round(time.time()-start,2)),
							  '-----','Loss: %.2f' % mean_loss,', Variable usage = %.2f%%' % (100 * variable_usage)) # ,
							  # '|||','%.3f' % loss_crit,'%.3f' % variable_usage_crit)

				elif (it - best_it) == lookback * check_every:
					if verbose:
						print('EARLY STOP: Lam={}, Iter {}'.format(lam,it + 1))
					break

		# Restore best model.
		restore_parameters(vmlp, best_model)


		if not os.path.exists(gc_dir):
			os.mkdir(gc_dir)
		if not os.path.exists(os.path.join(gc_dir,dir_name)):
			os.mkdir(os.path.join(gc_dir,dir_name))

		file_name = '{}.trial{}.lam{}.ridge{}.h{}.{}.lag{}.{}'.format(method,trial_no,lam,
					lam_ridge,hidden[0],penalty,lag,dynamics)
		GC_lag = vmlp.GC(threshold=False, ignore_lag=False).cpu()
		torch.save(GC_lag, os.path.join(gc_dir,dir_name,file_name + '.pt'))

		GC_lag = vmlp.GC(threshold=False, ignore_lag=True).cpu()
		torch.save(GC_lag, os.path.join(gc_dir,dir_name,file_name + '.ignore_lag.pt'))





#!/usr/bin/env python

### Authors: Anish Mudide (amudide), Alex Wu (alexw16), Rohit Singh (rs239)
### 2022
### MIT Licence
###

import os
import numpy as np
import scanpy as sc
import argparse
import time
import ray
from ray import tune
import statistics
import scvelo as scv

from .models import *
from .train import *
from .utils import *

def execute_cmdline():

	parser = argparse.ArgumentParser()
	parser.add_argument('-m','--method',dest='method',type=str,default='baseline')
	parser.add_argument('-ds','--dataset',dest='dataset',type=str)
	parser.add_argument('-dyn','--dyn',dest='dynamics',type=str,default='pseudotime')
	parser.add_argument('-dev','--device',dest='device',type=str,default='cpu')
	parser.add_argument('-s','--seed',dest='seed',type=int,default=0,help='random seed used for trial. set to 0,1,2 etc.')
	parser.add_argument('-lmr','--lam_ridge',dest='lam_ridge',type=float,default=0., help='Unsupported currently')
	parser.add_argument('-p','--penalty',dest='penalty',type=str,default="H")
	parser.add_argument('-l','--lag',dest='lag',type=int,default=5)
	parser.add_argument('-hd', '--hidden',dest='hidden',type=int,default=16)
	parser.add_argument('-mi','--max_iter',dest='max_iter',type=int,default=500)
	parser.add_argument('-lr','--learning_rate',dest='learning_rate',type=float,default=0.0001)
	parser.add_argument('-pr','--proba',dest='proba',type=int,default=1)
	parser.add_argument('-tol','--tolerance',dest='tolerance',type=float,default=0.01)
	parser.add_argument('-ce','--check_every',dest='check_every',type=int,default=100)
	parser.add_argument('-rd','--root_dir',dest='root_dir',type=str)
	parser.add_argument('-ls','--lam_start',dest='lam_start',type=float,default=-1)
	parser.add_argument('-le','--lam_end',dest='lam_end',type=float,default=1)
	parser.add_argument('-xn','--x_norm',dest='x_norm',type=str,default='none',choices=['none','zscore','to_count:zscore'])
	parser.add_argument('-nl','--num_lambdas',dest='num_lambdas',type=int,default=19)

	args = parser.parse_args()

	data_dir = os.path.join(args.root_dir,'datasets','preprocessed')
	gc_dir = os.path.join(args.root_dir,'results',args.dataset,'gc')

	if not os.path.exists(os.path.join(args.root_dir,'results',args.dataset)):
		os.mkdir(os.path.join(args.root_dir,'results',args.dataset))
	if not os.path.exists(gc_dir):
		os.mkdir(gc_dir)

	adata = sc.read(os.path.join(data_dir,'{}.h5ad'.format(args.dataset)))

	if args.dynamics == 'pseudotime':
		print('Inferring pseudotime transition matrix...')
		sc.tl.pca(adata, svd_solver='arpack')
		A = construct_dag(adata.obsm['X_pca'], adata.uns['iroot'])
		A = A.T
		A = construct_S(torch.FloatTensor(A))

	elif args.dynamics == 'rna_velocity':
		print('Inferring RNA velocity transition matrix...')

		scv.pp.moments(adata, n_pcs=30, n_neighbors=30)
		scv.tl.velocity(adata)
		scv.tl.velocity_graph(adata)
		vk = VelocityKernel(adata).compute_transition_matrix()
		A = vk.transition_matrix
		A = A.toarray()
		for i in range(len(A)):
			for j in range(len(A)):
				if A[i][j] > 0 and A[j][i] > 0 and A[i][j] > A[j][i]:
					A[j][i] = 0

		# if proba is False (0), it won't use the probabilistic 
		# transition matrix
		if not args.proba:
			for i in range(len(A)):
				nzeros = []
				for j in range(len(A)):
					if A[i][j] > 0:
						nzeros.append(A[i][j])
				m = statistics.median(nzeros)
				for j in range(len(A)):
					if A[i][j] < m:
						A[i][j] = 0
					else:
						A[i][j] = 1

		A = construct_S(torch.FloatTensor(A))

	if args.x_norm == 'zscore':

		print('Normalizing data: 0 mean, 1 SD')
		X_orig = adata[:,adata.var['is_reg']].X.toarray().copy()
		std = X_orig.std(0)
		std[std == 0] = 1
		X = torch.FloatTensor(X_orig-X_orig.mean(0))/std

		Y_orig = adata[:,adata.var['is_target']].X.toarray().copy()
		std = Y_orig.std(0)
		std[std == 0] = 1
		Y = torch.FloatTensor(Y_orig-Y_orig.mean(0))/std	

	elif args.x_norm == 'to_count:zscore':

		print('Use counts: 0 mean, 1 SD')
		X_orig = adata[:,adata.var['is_reg']].X.toarray().copy()
		X_orig = 2**X_orig-1
		std = X_orig.std(0)
		std[std == 0] = 1
		X = torch.FloatTensor(X_orig-X_orig.mean(0))/std

		Y_orig = adata[:,adata.var['is_target']].X.toarray().copy()
		Y_orig = 2**Y_orig-1
		std = Y_orig.std(0)
		std[std == 0] = 1
		Y = torch.FloatTensor(Y_orig-Y_orig.mean(0))/std

	else:

		assert args.x_norm == 'none'				
		X = torch.FloatTensor(adata[:,adata.var['is_reg']].X.toarray())
		Y = torch.FloatTensor(adata[:,adata.var['is_target']].X.toarray())


	print('# of Regs: {}, # of Targets: {}'.format(X.shape[1],Y.shape[1]))

	print('Performing diffusion...')
	A = torch.FloatTensor(A)
	AX = calculate_AX(A,X,args.lag)

	dir_name = '{}.seed{}.h{}.{}.lag{}.{}'.format(args.method,args.seed,args.hidden,args.penalty,args.lag,args.dynamics)

	if not os.path.exists(os.path.join(gc_dir,dir_name)):
		os.mkdir(os.path.join(gc_dir,dir_name))

	ray.init(object_store_memory=10**9)

	total_start = time.time()
	lam_list = np.round(np.logspace(args.lam_start, args.lam_end, num=args.num_lambdas),4).tolist()
	# lam_list = sorted(list(set(lam_list)))

	config = {'method': args.method,
			  'AX': AX,
			  'Y': Y,
			  'trial': args.seed,
			  'lr': args.learning_rate,
			  'lam': tune.grid_search(lam_list),
			  'lam_ridge': args.lam_ridge,
			  'penalty': args.penalty,
			  'lag': args.lag,
			  'hidden': [args.hidden],
			  'max_iter': args.max_iter,
			  'device': args.device,
			  'lookback': 5,
			  'check_every': args.check_every,
			  'verbose': True,
			  'tol': args.tolerance,
			  'dynamics': args.dynamics,
			  'gc_dir': gc_dir,
			  'dir_name': dir_name}

	resources_per_trial = {"cpu": 1, "gpu": 0.2, "memory": 2 * 1024 * 1024 * 1024}
	analysis = tune.run(train_model,resources_per_trial=resources_per_trial,config=config,
						local_dir=os.path.join(args.root_dir,'results'))
	
	print('Total time:',time.time()-total_start)
	np.savetxt(os.path.join(gc_dir,dir_name + '.time.txt'),np.array([time.time()-total_start]))

if __name__ == "__main__":
	execute_cmdline()




from .models import VeloramaMLP
from .train import train_model
from .run import execute_cmdline

__all__ = ['VeloramaMLP', 'train_model', 'execute_cmdline']


import pkgutil, pathlib, importlib

# from pkgutil import iter_modules
# from pathlib import Path
# from importlib import import_module

# https://julienharbulot.com/python-dynamical-import.html
# iterate through the modules in the current package
#
# # package_dir = pathlib.Path(__file__).resolve().parent
# # for (_, module_name, _) in pkgutil.iter_modules([package_dir]):
# #     if 'datasets' in module_name:
# #         module = importlib.import_module(f"{__name__}.{module_name}")



#!/usr/bin/env python

### Authors: Anish Mudide (amudide), Alex Wu (alexw16), Rohit Singh (rs239)
### 2022
### MIT Licence
###
### Credit: parts of this code make use of the code from Tank et al.'s "Neural Granger Causality"
###    - https://github.com/iancovert/Neural-GC
###    - https://arxiv.org/abs/1802.05842

import torch
import torch.nn as nn

from .utils import construct_S, seq2dag, construct_dag, activation_helper

class MLP(nn.Module):

    def __init__(self, num_series, lag, hidden, device, activation):
        super(MLP, self).__init__()
        self.activation = activation_helper(activation)
        self.hidden = hidden
        self.lag = lag
        # self.T = T
        self.device = device

        # Set up network.
        layer = nn.Conv1d(num_series, hidden[0], lag)
        modules = [layer]

        for d_in, d_out in zip(hidden, hidden[1:] + [1]):
            layer = nn.Conv1d(d_in, d_out, 1)
            modules.append(layer)

        # Register parameters.
        self.layers = nn.ModuleList(modules)

    def forward(self,AX):
        ret = 0 # torch.zeros(self.T, self.hidden[0], device=self.device)
        for i in range(self.lag):
            ret = ret + torch.matmul(AX[i], self.layers[0].weight[:, :, self.lag - 1 - i].T)
        ret = ret + self.layers[0].bias

        ret = ret.T
        for i, fc in enumerate(self.layers):
            if i == 0:
                continue
            ret = self.activation(ret)
            ret = fc(ret)

        ret = ret.T

        ret = torch.unsqueeze(ret, 0)
        
        return ret

class VeloramaMLP(nn.Module):
    def __init__(self, num_targets, num_series, lag, hidden, device, activation='relu'):
        '''
        VeloramaMLP model with one MLP per time series.

        Args:
          num_series: dimensionality of multivariate time series (# of regulators).
          lag: number of previous time points to use in prediction.
          hidden: list of number of hidden units per layer.
          activation: nonlinearity at each layer.
        '''
        super(VeloramaMLP, self).__init__()
        self.p = num_series
        self.lag = lag
        self.activation = activation_helper(activation)

        # Set up networks.
        self.networks = nn.ModuleList([
            MLP(num_series, lag, hidden, device, activation)
            for _ in range(num_targets)])

    def forward(self,AX):
        '''
        Perform forward pass.

        Args:
          X: torch tensor of shape (batch, T, p).
        '''
        return torch.cat([network(AX) for network in self.networks], dim=2).squeeze()

    def GC(self, threshold=True, ignore_lag=True):
        '''
        Extract learned Granger causality.

        Args:
          threshold: return norm of weights, or whether norm is nonzero.
          ignore_lag: if true, calculate norm of weights jointly for all lags.

        Returns:
          GC: (p x p) or (p x p x lag) matrix. In first case, entry (i, j)
            indicates whether variable j is Granger causal of variable i. In
            second case, entry (i, j, k) indicates whether it's Granger causal
            at lag k.
        '''
        if ignore_lag:
            GC = [torch.norm(net.layers[0].weight, dim=(0, 2))
                  for net in self.networks]
        else:
            GC = [torch.norm(net.layers[0].weight, dim=0)
                  for net in self.networks]
        GC = torch.stack(GC)
        if threshold:
            return (GC > 0).int()
        else:
            return GC

def prox_update(network, lam, lr, penalty):
    '''
    Perform in place proximal update on first layer weight matrix.

    Args:
      network: MLP network.
      lam: regularization parameter.
      lr: learning rate.
      penalty: one of GL (group lasso), GSGL (group sparse group lasso),
        H (hierarchical).
    '''
    W = network.layers[0].weight
    hidden, p, lag = W.shape
    if penalty == 'GL':
        norm = torch.norm(W, dim=(0, 2), keepdim=True)
        W.data = ((W / torch.clamp(norm, min=(lr * lam)))
                  * torch.clamp(norm - (lr * lam), min=0.0))
    elif penalty == 'GSGL':
        norm = torch.norm(W, dim=0, keepdim=True)
        W.data = ((W / torch.clamp(norm, min=(lr * lam)))
                  * torch.clamp(norm - (lr * lam), min=0.0))
        norm = torch.norm(W, dim=(0, 2), keepdim=True)
        W.data = ((W / torch.clamp(norm, min=(lr * lam)))
                  * torch.clamp(norm - (lr * lam), min=0.0))
    elif penalty == 'H':
        # Lowest indices along third axis touch most lagged values.
        for i in range(lag):
            norm = torch.norm(W[:, :, :(i + 1)], dim=(0, 2), keepdim=True)
            W.data[:, :, :(i+1)] = (
                (W.data[:, :, :(i+1)] / torch.clamp(norm, min=(lr * lam)))
                * torch.clamp(norm - (lr * lam), min=0.0))
    else:
        raise ValueError('unsupported penalty: %s' % penalty)


def regularize(network, lam, penalty):
    '''
    Calculate regularization term for first layer weight matrix.

    Args:
      network: MLP network.
      penalty: one of GL (group lasso), GSGL (group sparse group lasso),
        H (hierarchical).
    '''
    W = network.layers[0].weight
    hidden, p, lag = W.shape
    if penalty == 'GL':
        return lam * torch.sum(torch.norm(W, dim=(0, 2)))
    elif penalty == 'GSGL':
        return lam * (torch.sum(torch.norm(W, dim=(0, 2)))
                      + torch.sum(torch.norm(W, dim=0)))
    elif penalty == 'H':
        # Lowest indices along third axis touch most lagged values.
        return lam * sum([torch.sum(torch.norm(W[:, :, :(i+1)], dim=(0, 2)))
                          for i in range(lag)])
    else:
        raise ValueError('unsupported penalty: %s' % penalty)


def ridge_regularize(network, lam):
    '''Apply ridge penalty at all subsequent layers.'''
    return lam * sum([torch.sum(fc.weight ** 2) for fc in network.layers[1:]])


def restore_parameters(model, best_model):
    '''Move parameter values from best_model to model.'''
    for params, best_params in zip(model.parameters(), best_model.parameters()):
        params.data = best_params



from setuptools import setup

setup(name='velorama',
      version='0.0.1',
      description='Gene regulatory network inference for RNA velocity and pseudotime data',
      url='http://github.com/rs239/velorama',
      author='Anish Mudide, Alex Wu, Rohit Singh',
      author_email='rsingh@alum.mit.edu',
      license='MIT',
      packages=['velorama'],
      install_requires = 'numpy,scipy,pandas,sklearn,cellrank,scvelo,ray'.split(','),
      scripts = ['bin/velorama'],
      zip_safe=False)
